from frozendict import frozendict
import copy

d = dict(a=1, b=2)
fd = frozendict(a=1, b=2)
reveal_type(d["a"])
reveal_type(fd["a"])
reveal_type(iter(d))
reveal_type(iter(fd))
reveal_type(dict.fromkeys("abc", 0))
reveal_type(frozendict.fromkeys("abc", 0))
reveal_type(d.copy())
reveal_type(fd.copy())
reveal_type(tuple(d.items())[0])
reveal_type(fd.item(0))
reveal_type(tuple(d.keys())[0])
reveal_type(fd.key(0))
reveal_type(tuple(d.values())[0])
reveal_type(fd.value(0))
reveal_type(d.get("a"))
reveal_type(fd.get("a"))
reveal_type(d.items())
reveal_type(fd.items())
reveal_type(d.keys())
reveal_type(fd.keys())
reveal_type(d.values())
reveal_type(fd.values())
reveal_type(copy.copy(d))
reveal_type(copy.copy(fd))
reveal_type(copy.deepcopy(d))
reveal_type(copy.deepcopy(fd))

reveal_type(reversed(fd))
reveal_type(fd | {1: 2})
reveal_type(fd.setdefault("a", 0))
reveal_type(fd.set("abc", 1))
reveal_type(fd.delete("a"))
